"""
discord.sinks
~~~~~~~~~~~~~

A place to store all officially given voice sinks.

:copyright: 2021-present Pycord Development
:license: MIT, see LICENSE for more details.
"""
from .core import *
from .errors import *
from .m4a import *
from .mka import *
from .mkv import *
from .mp3 import *
from .mp4 import *
from .ogg import *
from .pcm import *
from .wave import *
